#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jan 22 11:23:50 2019

@author: nmp
"""

import socket
import struct
from antidotedb.proto import *

class AntidoteException(Exception):
    pass


#---------------------------------------------------------------------------------
# Classes that represent data objects stores in Antidote
#---------------------------------------------------------------------------------
class Counter :
    """ Represents a counter object
    """
    def __init__(self, key, val):
        self.key = key
        self.val = val

    def value( self) :
        return self.val

    def incOp( self, val) :
        return Counter.IncOp( self.key, val)

    def incKeyOp( val, key) :
        return Counter.IncOp( key, val)

    def __repr__( self):
        return self.key.data_type_name + ' { val : ' + str(self.val) + '}'

    class IncOp:
        def __init__(self, key, val, data_type = None):
            self.val = val
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.counterop.inc = self.val

    def resetOp( self) :
        return Counter.ResetOp( self.key)

    # def resetKeyOp(key) :
    #     return Counter.ResetOp( key)

    class ResetOp:
        def __init__(self, key, data_type = None):
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.resetop.SetInParent()


class Flag :
    """ Represents a register object
    """
    def __init__(self, key, val):
        self.key = key
        self.val = val

    def value( self) :
        return self.val

    def enableOp( self) :
        return Flag.UpdateOp( self.key, True)

    # def enableKeyOp( key) :
    #     return Flag.UpdateOp( key, True)

    def disableOp( self) :
        return Flag.UpdateOp( self.key, False)

    # def disableKeyOp( key) :
    #     return Flag.UpdateOp( key, False)

    # def updateKeyOp( val, key) :
    #     return Flag.UpdateOp( key, val)

    class UpdateOp:
        def __init__(self, key, val, data_type = None):
            self.val = val
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.flagop.value = self.val

    def resetOp( self) :
        return Flag.ResetOp( self.key)

    # def resetKeyOp(key) :
    #     return Flag.ResetOp( key)

    class ResetOp:
        def __init__(self, key, data_type = None):
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.resetop.SetInParent()


    def __repr__( self):
        return self.key.data_type_name + ' { val : ' + str(self.val) + '}'


class Register :
    """ Represents a register object
    """
    def __init__(self, key, val):
        self.key = key
        self.val = val

    def value( self) :
        return self.val

    def assignOp( self, val) :
        return Register.AssignOp( self.key, val)

    # def assignKeyOp( val, key) :
    #     return Register.AssignOp( key, val)

    class AssignOp:
        def __init__(self, key, val, data_type = None):
            self.val = val
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.regop.value = self.val

    def resetOp( self) :
        return Register.ResetOp( self.key)

    # def resetKeyOp(key) :
    #     return Register.ResetOp( key)

    class ResetOp:
        def __init__(self, key, data_type = None):
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.resetop.SetInParent()

    def __repr__( self):
        return self.key.data_type_name + ' { val : ' + str(self.val) + '}'


class MVRegister :
    """ Represents a MV register object
    """
    def __init__(self, key, vals):
        self.key = key
        self.vals = vals

    def values( self) :
        return self.vals

    def assignOp( self, val) :
        return MVRegister.AssignOp( self.key, val)

    def assignKeyOp( val, key) :
        return MVRegister.AssignOp( key, val)

    class AssignOp:
        def __init__(self, key, val, data_type = None):
            self.val = val
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.regop.value = self.val

    def resetOp( self) :
        return MVRegister.ResetOp( self.key)

    # def resetKeyOp(key) :
    #     return MVRegister.ResetOp( key)

    class ResetOp:
        def __init__(self, key, data_type = None):
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.resetop.SetInParent()

    def __repr__( self):
        return self.key.data_type_name + ' { val : ' + str(self.vals) + '}'

class Set :
    """ Represents a Set
    """
    def __init__(self, key, vals):
        self.key = key
        self.vals = vals

    def values( self) :
        return self.vals

    def addOp( self, val) :
        return Set.AddOp( self.key, val)

    # def addKeyOp( val, key) :
    #     return Set.AddOp( key, val)

    class AddOp:
        def __init__(self, key, val, data_type = None):
            self.val = val
            self.key = key
            if type( self.val) != list:
                self.val = [self.val]
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.setop.optype = ApbSetUpdate.SetOpType.Value('ADD')
            op.setop.adds.extend( self.val)

    def removeOp( self, val) :
        return Set.RemoveOp( self.key, val)

    # def removeKeyOp( val, key) :
    #     return Set.RemoveOp( key, val)

    class RemoveOp:
        def __init__(self, key, val, data_type = None):
            self.val = val
            self.key = key
            if type( self.val) != list:
                self.val = [self.val]
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.setop.optype = ApbSetUpdate.SetOpType.Value('REMOVE')
            op.setop.rems.extend( self.val)

    def resetOp( self) :
        return Set.ResetOp( self.key)

    # def resetKeyOp(key) :
    #     return Set.ResetOp( key)

    class ResetOp:
        def __init__(self, key, data_type = None):
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.resetop.SetInParent()

    def __repr__( self):
        return self.key.data_type_name + ' { val : ' + str(self.vals) + '}'

class Map :
    """ Represents a Set
    """
    def __init__(self, key, vals):
        self.key = key
        self.vals = vals

    def values( self) :
        return self.vals

    def updateOp( self, upds) :
        return Map.UpdateOp( self.key, upds)

    # def updateKeyOp( upds, key) :
    #     return Map.UpdateOp( key, upds)

    class UpdateOp:
        def __init__(self, key, upds, data_type = None):
            self.upds = upds
            self.key = key
            if type( self.upds) != list:
                self.upds = [self.upds]
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, opbase) :
            for upd in self.upds :
                op = opbase.mapop.updates.add()
                op.key.key = upd.getKey()
                op.key.type = upd.getType()
                upd.recordOp( op.update)

    def removeOp( self, keys) :
        return Map.RemoveOp( self.key, keys)

    # def removeKeyOp( upds, keys) :
    #     return Map.RemoveOp( key, keys)

    class RemoveOp:
        def __init__(self, key, keys, data_type = None):
            self.keys = keys
            self.key = key
            if type( self.keys) != list:
                self.keys = [self.keys]
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, opbase) :
            for opkey in self.keys :
                op = opbase.mapop.removedKeys.add()
                op.key = opkey.key
                op.type = opkey.data_type

    def resetOp( self) :
        return Map.ResetOp( self.key)

    # def resetKeyOp(key) :
    #     return Map.ResetOp( key)

    class ResetOp:
        def __init__(self, key, data_type = None):
            self.key = key
            if data_type == None :
                self.data_type = key.data_type
            else:
                self.data_type = CRDT_type.Value(data_type)

        def getType( self) :
            return self.data_type

        def getKey( self) :
            return self.key

        def recordOp( self, op) :
            op.resetop.SetInParent()

    def __repr__( self):
        return self.key.data_type_name + str(self.vals)


#---------------------------------------------------------------------------------
# Utilitiles
#---------------------------------------------------------------------------------
class ServerAddress:
    """ Records the information about a server endpoint, consisting of
    IP address and port"""
    def __init__(self, address, port):
        """ Address should be a string and port an integer """
        self.address = address
        self.port = port

    def getAddress( self) :
        """ Returns the address as a tuple """
        return (self.address, self.port)

#---------------------------------------------------------------------------------
# Antidote client main classes
#---------------------------------------------------------------------------------
class Key :
    """" Class for an interative transaction """
    def __init__(self, bucket, key, data_type):
        if type(bucket) == bytes :
            self.bucket = bucket
        else:
            self.bucket = bytes(bucket, 'utf-8')
        if type(key) == bytes :
            self.key = key
        else:
            self.key = bytes(key, 'utf-8')
        if type(data_type) == int :
            self.data_type = data_type
            self.data_type_name = CRDT_type.Name(data_type)
        else :
            self.data_type = CRDT_type.Value(data_type)
            self.data_type_name = data_type

class StaticTransaction :
    """" Class for an interative transaction """
    def __init__(self, clt, red_blue, min_snapshot):
        self.antidoteClient = clt
        self.red_blue = red_blue
        self.min_snapshot = min_snapshot


    def read_objects_raw( self, keys) :
        """ Execute one or more read object operations
        """
        op = ApbStaticReadObjects()
        op.transaction.properties.read_write = 1
        op.transaction.properties.red_blue = self.red_blue
        if self.min_snapshot is not None:
            op.transaction.timestamp = self.min_snapshot
        if type( keys) != list :
            keys= [keys]
        for key in keys :
            obj = op.objects.add()
            obj.key = key.key
            obj.bucket = key.bucket
            obj.type = key.data_type
        self.antidoteClient.sendMessageStaticReadObjects( op)
        res = self.antidoteClient.recvMessageStaticReadObjectsResp()
        return res

    def read_objects( self, keys) :
        """ Execute one or more read object operations
        """
        op = ApbStaticReadObjects()
        op.transaction.properties.read_write = 1
        op.transaction.properties.red_blue = self.red_blue
        if self.min_snapshot is not None:
            op.transaction.timestamp = self.min_snapshot
        if type( keys) != list :
            keys= [keys]
        for key in keys :
            obj = op.objects.add()
            obj.key = key.key
            obj.bucket = key.bucket
            obj.type = key.data_type
        self.antidoteClient.sendMessageStaticReadObjects( op)
        res = self.antidoteClient.recvMessageStaticReadObjectsResp()
        if res.objects.success == True:
            reply = []
            for i in range(len(keys)):
                key = keys[i]
                val = res.objects.objects[i]
                reply.append( AntidoteClient.decodeObjectReply( key, val))
            return reply
        else:
            return None

    def update_objects( self, updates) :
        """ Execute one or more read object operations
        """
        op = ApbStaticUpdateObjects()
        op.transaction.properties.read_write = 2
        op.transaction.properties.red_blue = self.red_blue
        if self.min_snapshot is not None:
            op.transaction.timestamp = self.min_snapshot
        if type( updates) != list :
            updates = [updates]
        for upd in updates :
            opupd = op.updates.add()
            key = upd.getKey()
            opupd.boundobject.key = key.key
            opupd.boundobject.bucket = key.bucket
            opupd.boundobject.type = key.data_type
            upd.recordOp( opupd.operation)
        #print(op)
        self.antidoteClient.sendMessageStaticUpdateObjects( op)
        res = self.antidoteClient.recvMessageCommitResp()
        if res.success == True:
            self.antidoteClient.last_commit = res.commit_time
        return res.success == True




class InteractiveTransaction :
    """" Class for an interative transaction """
    def __init__(self, clt, txDescriptor):
        self.antidoteClient = clt
        self.txDescriptor = txDescriptor


    def read_objects_raw( self, keys) :
        """ Execute one or more read object operations
        """
        op = ApbReadObjects()
        op.transaction_descriptor = self.txDescriptor
        if type( keys) != list :
            keys= [keys]
        for key in keys :
            obj = op.boundobjects.add()
            obj.key = key.key
            obj.bucket = key.bucket
            obj.type = key.data_type
        self.antidoteClient.sendMessageReadObjects( op)
        res = self.antidoteClient.recvMessageReadObjectsResp()
        return res

    def read_objects( self, keys) :
        """ Execute one or more read object operations
        """
        op = ApbReadObjects()
        op.transaction_descriptor = self.txDescriptor
        if type( keys) != list :
            keys= [keys]
        for key in keys :
            obj = op.boundobjects.add()
            obj.key = key.key
            obj.bucket = key.bucket
            obj.type = key.data_type
        self.antidoteClient.sendMessageReadObjects( op)
        res = self.antidoteClient.recvMessageReadObjectsResp()
        if res.success == True:
            reply = []
            for i in range(len(keys)):
                key = keys[i]
                val = res.objects[i]
                reply.append( AntidoteClient.decodeObjectReply( key, val))
            return reply
        else:
            return None

    def update_objects( self, updates) :
        """ Execute one or more read object operations
        """
        op = ApbUpdateObjects()
        op.transaction_descriptor = self.txDescriptor
        if type( updates) != list :
            updates = [updates]
        for upd in updates :
            opupd = op.updates.add()
            key = upd.getKey()
            opupd.boundobject.key = key.key
            opupd.boundobject.bucket = key.bucket
            opupd.boundobject.type = key.data_type
            upd.recordOp( opupd.operation)
        self.antidoteClient.sendMessageUpdateObjects( op)
        res = self.antidoteClient.recvMessageOperationResp()
        return res.success == True

    def commit( self) :
        """ Commit transaction. Commit time gets recoded in AntidoteClient as
        last_commit
        """
        op = ApbCommitTransaction()
        op.transaction_descriptor = self.txDescriptor
        self.antidoteClient.sendMessageCommitTransaction( op)
        res = self.antidoteClient.recvMessageCommitResp()
        if res.success == True:
            self.antidoteClient.last_commit = res.commit_time
        return res.success == True

    def abort( self) :
        """ Commit transaction. Commit time gets recoded in AntidoteClient as
        last_commit
        """
        op = ApbAbortTransaction()
        op.transaction_descriptor = self.txDescriptor
        self.antidoteClient.sendMessageCommitTransaction( op)
        res = self.antidoteClient.recvMessageCommitResp()
        return res





class AntidoteClient :
    """ AntidoteDB protobuf client
    """
    def __init__(self, address = None, port = None, serverAddress = None):
        """ Intiialize a client, connecting to an AntidoteDB server located
        at serverAddress (of type ServerAddress)
        """
        if serverAddress == None:
            self.connect( ServerAddress( address, port))
        else:
            self.connect( serverAddress)
        self.last_commit = None

    def connect( self, serverAddress):
        """ Connect this AntidoteClient to the AntidoteDB server located
        at serverAddress (of type ServerAddress)
        """
        self.serverAddress = serverAddress
        try :
            self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.server.connect( serverAddress.getAddress())
        except socket.error as e:
            self.server = None
            raise AntidoteException( e)

    def close( self) :
        """ Closes the underlying connection of this client
        """
        try :
            if self.server != None :
                self.server.close()
                self.server = None
        except socket.error as e:
            self.server = None
            raise AntidoteException( e)

    def start_transaction( self, read_write = 0, red_blue = 0, min_snapshot = None) :
        """ Starts an interactive transaction, returning an object of type
        InteractiveTransaction. Raises an exception if the operation fails.
        """
        apbtxn = ApbStartTransaction()
        apbtxn.properties.read_write = read_write
        apbtxn.properties.red_blue = red_blue
        if min_snapshot is not None:
            apbtxn.timestamp = min_snapshot
        self.sendMessageStartTransaction( apbtxn)
        apbtxnresp = self.recvMessageStartTransactionResp()
        if apbtxnresp.success :
            return InteractiveTransaction( self, apbtxnresp.transaction_descriptor)
        else :
            raise AntidoteException( "Start transaction failed")

    def start_static_transaction( self, red_blue = 0, min_snapshot = None) :
        """ Starts a static transaction, returning an object of type
        StaticTransaction. Raises an exception if the operation fails.
        """
        return StaticTransaction( self, red_blue, min_snapshot)


    #---------------------------------------------------------------------------------
    # Utilities
    #---------------------------------------------------------------------------------
    def decodeObjectReply( key, val) :
        """ Decodes the value of an object returned in a read, creating
        the appropriate data type
        """
        if key.data_type == CRDT_type.Value("COUNTER") or key.data_type == CRDT_type.Value("FATCOUNTER"):
            return Counter(key, val.counter.value)
        if key.data_type == CRDT_type.Value("LWWREG"):
            return Register(key, val.reg.value)
        if key.data_type == CRDT_type.Value("MVREG"):
            return MVRegister(key, val.mvreg.values)
        if key.data_type == CRDT_type.Value("ORSET") or key.data_type == CRDT_type.Value("RWSET"):
            return Set(key, val.set.value)
        if key.data_type == CRDT_type.Value("FLAG_EW") or key.data_type == CRDT_type.Value("FLAG_DW"):
            return Flag(key, val.flag.value)
        if key.data_type == CRDT_type.Value("GMAP") or key.data_type == CRDT_type.Value("RRMAP"):
            els = {}
            for el in val.map.entries:
                els[el.key.key] = AntidoteClient.decodeObjectReply( Key( "", el.key.key, el.key.type), el.value)
            return Map(key, els)
        raise AntidoteException( "Unknown type")

    #---------------------------------------------------------------------------------
    # Methods for reading messages from the server
    #---------------------------------------------------------------------------------
    def recvNBytes( self, nbytes) :
        if self.server == None:
            raise AntidoteException( "Not connected to server")
        try :
#            data = []
#            nRecv = 0;
#            while nRecv < nbytes:
#                d = self.server.recv( nbytes - nRecv)
#                nRecv = nRecv + len(d)
#                data.append( d)
#            return ''.join(data)
            return self.server.recv( nbytes)
        except socket.error as e:
            raise AntidoteException( e)

    def recvMessageRaw( self) :
        data = self.recvNBytes( 5)
        msgSize, msgCode = struct.unpack( ">IB", data)
        return msgCode, self.recvNBytes( msgSize - 1)

    def recvMessageCommitResp( self) :
        msgCode, data = self.recvMessageRaw()
        if msgCode == 127 :
            msg = ApbCommitResp()
            msg.ParseFromString(data)
            return msg
        raise AntidoteClient.invalidReply(msgCode, data)

    def recvMessageOperationResp( self) :
        msgCode, data = self.recvMessageRaw()
        if msgCode == 111 :
            msg = ApbOperationResp()
            msg.ParseFromString(data)
            return msg
        raise AntidoteClient.invalidReply(msgCode, data)

    def recvMessageReadObjectsResp( self) :
        msgCode, data = self.recvMessageRaw()
        if msgCode == 126 :
            msg = ApbReadObjectsResp()
            msg.ParseFromString(data)
            return msg
        raise AntidoteClient.invalidReply(msgCode, data)

    def recvMessageStaticReadObjectsResp( self) :
        msgCode, data = self.recvMessageRaw()
        if msgCode == 128 :
            msg = ApbStaticReadObjectsResp()
            msg.ParseFromString(data)
            return msg
        raise AntidoteClient.invalidReply(msgCode, data)

    def recvMessageStartTransactionResp( self) :
        msgCode, data = self.recvMessageRaw()
        if msgCode == 124 :
            msg = ApbStartTransactionResp()
            msg.ParseFromString(data)
            return msg
        raise AntidoteClient.invalidReply(msgCode, data)

    @staticmethod
    def invalidReply(msgCode, data):
        if msgCode == 0:
            msg = ApbErrorResp()
            msg.ParseFromString(data)
            return AntidoteException(f"Antidote error {msg.errcode}: {msg.errmsg.decode('utf-8')}")
        return AntidoteException(f"Invalid reply (code {msgCode})")

    #---------------------------------------------------------------------------------
    # Methods for sending messages to the server
    #---------------------------------------------------------------------------------
    def sendMessage( self, msg, msgCode) :
        if self.server == None:
            raise AntidoteException( "Not connected to server")
        try :
            msgB = msg.SerializeToString()
            buf = struct.pack( '>IB', len(msgB) + 1, msgCode)
            self.server.sendall( buf)
            self.server.sendall( msg.SerializeToString())
        except socket.error as e:
            raise AntidoteException( e)

    def sendMessageReadObjects( self, msg) :
        self.sendMessage( msg, 116)

    def sendMessageUpdateObjects( self, msg) :
        self.sendMessage( msg, 118)

    def sendMessageStartTransaction( self, msg) :
        self.sendMessage( msg, 119)

    def sendMessageAbortTransaction( self, msg) :
        self.sendMessage( msg, 120)

    def sendMessageCommitTransaction( self, msg) :
        self.sendMessage( msg, 121)

    def sendMessageStaticUpdateObjects( self, msg) :
        self.sendMessage( msg, 122)

    def sendMessageStaticReadObjects( self, msg) :
        self.sendMessage( msg, 123)




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: antidote.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='antidote.proto',
  package='',
  syntax='proto2',
  serialized_options=b'\n\030eu.antidotedb.antidotepbB\nAntidotePB',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0e\x61ntidote.proto\"/\n\x0c\x41pbErrorResp\x12\x0e\n\x06\x65rrmsg\x18\x01 \x02(\x0c\x12\x0f\n\x07\x65rrcode\x18\x02 \x02(\r\"\x1f\n\x10\x41pbCounterUpdate\x12\x0b\n\x03inc\x18\x01 \x01(\x12\"\"\n\x11\x41pbGetCounterResp\x12\r\n\x05value\x18\x01 \x02(\x11\"u\n\x0c\x41pbSetUpdate\x12\'\n\x06optype\x18\x01 \x02(\x0e\x32\x17.ApbSetUpdate.SetOpType\x12\x0c\n\x04\x61\x64\x64s\x18\x02 \x03(\x0c\x12\x0c\n\x04rems\x18\x03 \x03(\x0c\" \n\tSetOpType\x12\x07\n\x03\x41\x44\x44\x10\x01\x12\n\n\x06REMOVE\x10\x02\"\x1e\n\rApbGetSetResp\x12\r\n\x05value\x18\x01 \x03(\x0c\"\x1d\n\x0c\x41pbRegUpdate\x12\r\n\x05value\x18\x01 \x02(\x0c\"\x1e\n\rApbGetRegResp\x12\r\n\x05value\x18\x01 \x02(\x0c\"!\n\x0f\x41pbGetMVRegResp\x12\x0e\n\x06values\x18\x01 \x03(\x0c\"2\n\tApbMapKey\x12\x0b\n\x03key\x18\x01 \x02(\x0c\x12\x18\n\x04type\x18\x02 \x02(\x0e\x32\n.CRDT_type\"U\n\x0c\x41pbMapUpdate\x12$\n\x07updates\x18\x01 \x03(\x0b\x32\x13.ApbMapNestedUpdate\x12\x1f\n\x0bremovedKeys\x18\x02 \x03(\x0b\x32\n.ApbMapKey\"R\n\x12\x41pbMapNestedUpdate\x12\x17\n\x03key\x18\x01 \x02(\x0b\x32\n.ApbMapKey\x12#\n\x06update\x18\x02 \x02(\x0b\x32\x13.ApbUpdateOperation\".\n\rApbGetMapResp\x12\x1d\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x0c.ApbMapEntry\"I\n\x0b\x41pbMapEntry\x12\x17\n\x03key\x18\x01 \x02(\x0b\x32\n.ApbMapKey\x12!\n\x05value\x18\x02 \x02(\x0b\x32\x12.ApbReadObjectResp\"\x1e\n\rApbFlagUpdate\x12\r\n\x05value\x18\x01 \x02(\x08\"\x1f\n\x0e\x41pbGetFlagResp\x12\r\n\x05value\x18\x01 \x02(\x08\"\x0e\n\x0c\x41pbCrdtReset\"6\n\x10\x41pbOperationResp\x12\x0f\n\x07success\x18\x01 \x02(\x08\x12\x11\n\terrorcode\x18\x02 \x01(\r\"g\n\x10\x41pbTxnProperties\x12\x12\n\nread_write\x18\x01 \x01(\r\x12\x10\n\x08red_blue\x18\x02 \x01(\r\x12\x14\n\x0cshared_locks\x18\x03 \x03(\x0c\x12\x17\n\x0f\x65xclusive_locks\x18\x04 \x03(\x0c\"G\n\x0e\x41pbBoundObject\x12\x0b\n\x03key\x18\x01 \x02(\x0c\x12\x18\n\x04type\x18\x02 \x02(\x0e\x32\n.CRDT_type\x12\x0e\n\x06\x62ucket\x18\x03 \x02(\x0c\"W\n\x0e\x41pbReadObjects\x12%\n\x0c\x62oundobjects\x18\x01 \x03(\x0b\x32\x0f.ApbBoundObject\x12\x1e\n\x16transaction_descriptor\x18\x02 \x02(\x0c\"[\n\x0b\x41pbUpdateOp\x12$\n\x0b\x62oundobject\x18\x01 \x02(\x0b\x32\x0f.ApbBoundObject\x12&\n\toperation\x18\x02 \x02(\x0b\x32\x13.ApbUpdateOperation\"\xd4\x01\n\x12\x41pbUpdateOperation\x12$\n\tcounterop\x18\x01 \x01(\x0b\x32\x11.ApbCounterUpdate\x12\x1c\n\x05setop\x18\x02 \x01(\x0b\x32\r.ApbSetUpdate\x12\x1c\n\x05regop\x18\x03 \x01(\x0b\x32\r.ApbRegUpdate\x12\x1c\n\x05mapop\x18\x05 \x01(\x0b\x32\r.ApbMapUpdate\x12\x1e\n\x07resetop\x18\x06 \x01(\x0b\x32\r.ApbCrdtReset\x12\x1e\n\x06\x66lagop\x18\x07 \x01(\x0b\x32\x0e.ApbFlagUpdate\"Q\n\x10\x41pbUpdateObjects\x12\x1d\n\x07updates\x18\x01 \x03(\x0b\x32\x0c.ApbUpdateOp\x12\x1e\n\x16transaction_descriptor\x18\x02 \x02(\x0c\"O\n\x13\x41pbStartTransaction\x12\x11\n\ttimestamp\x18\x01 \x01(\x0c\x12%\n\nproperties\x18\x02 \x01(\x0b\x32\x11.ApbTxnProperties\"5\n\x13\x41pbAbortTransaction\x12\x1e\n\x16transaction_descriptor\x18\x01 \x02(\x0c\"6\n\x14\x41pbCommitTransaction\x12\x1e\n\x16transaction_descriptor\x18\x01 \x02(\x0c\"b\n\x16\x41pbStaticUpdateObjects\x12)\n\x0btransaction\x18\x01 \x02(\x0b\x32\x14.ApbStartTransaction\x12\x1d\n\x07updates\x18\x02 \x03(\x0b\x32\x0c.ApbUpdateOp\"c\n\x14\x41pbStaticReadObjects\x12)\n\x0btransaction\x18\x01 \x02(\x0b\x32\x14.ApbStartTransaction\x12 \n\x07objects\x18\x02 \x03(\x0b\x32\x0f.ApbBoundObject\"]\n\x17\x41pbStartTransactionResp\x12\x0f\n\x07success\x18\x01 \x02(\x08\x12\x1e\n\x16transaction_descriptor\x18\x02 \x01(\x0c\x12\x11\n\terrorcode\x18\x03 \x01(\r\"\xcf\x01\n\x11\x41pbReadObjectResp\x12#\n\x07\x63ounter\x18\x01 \x01(\x0b\x32\x12.ApbGetCounterResp\x12\x1b\n\x03set\x18\x02 \x01(\x0b\x32\x0e.ApbGetSetResp\x12\x1b\n\x03reg\x18\x03 \x01(\x0b\x32\x0e.ApbGetRegResp\x12\x1f\n\x05mvreg\x18\x04 \x01(\x0b\x32\x10.ApbGetMVRegResp\x12\x1b\n\x03map\x18\x06 \x01(\x0b\x32\x0e.ApbGetMapResp\x12\x1d\n\x04\x66lag\x18\x07 \x01(\x0b\x32\x0f.ApbGetFlagResp\"]\n\x12\x41pbReadObjectsResp\x12\x0f\n\x07success\x18\x01 \x02(\x08\x12#\n\x07objects\x18\x02 \x03(\x0b\x32\x12.ApbReadObjectResp\x12\x11\n\terrorcode\x18\x03 \x01(\r\"H\n\rApbCommitResp\x12\x0f\n\x07success\x18\x01 \x02(\x08\x12\x13\n\x0b\x63ommit_time\x18\x02 \x01(\x0c\x12\x11\n\terrorcode\x18\x03 \x01(\r\"d\n\x18\x41pbStaticReadObjectsResp\x12$\n\x07objects\x18\x01 \x02(\x0b\x32\x13.ApbReadObjectsResp\x12\"\n\ncommittime\x18\x02 \x02(\x0b\x32\x0e.ApbCommitResp\"\x1c\n\x0b\x41pbCreateDC\x12\r\n\x05nodes\x18\x01 \x03(\t\"5\n\x0f\x41pbCreateDCResp\x12\x0f\n\x07success\x18\x01 \x02(\x08\x12\x11\n\terrorcode\x18\x02 \x01(\r\"\x1c\n\x1a\x41pbGetConnectionDescriptor\"X\n\x1e\x41pbGetConnectionDescriptorResp\x12\x0f\n\x07success\x18\x01 \x02(\x08\x12\x12\n\ndescriptor\x18\x02 \x01(\x0c\x12\x11\n\terrorcode\x18\x03 \x01(\r\"&\n\x0f\x41pbConnectToDCs\x12\x13\n\x0b\x64\x65scriptors\x18\x01 \x03(\x0c\"9\n\x13\x41pbConnectToDCsResp\x12\x0f\n\x07success\x18\x01 \x02(\x08\x12\x11\n\terrorcode\x18\x02 \x01(\r*\x92\x01\n\tCRDT_type\x12\x0b\n\x07\x43OUNTER\x10\x03\x12\t\n\x05ORSET\x10\x04\x12\n\n\x06LWWREG\x10\x05\x12\t\n\x05MVREG\x10\x06\x12\x08\n\x04GMAP\x10\x08\x12\t\n\x05RWSET\x10\n\x12\t\n\x05RRMAP\x10\x0b\x12\x0e\n\nFATCOUNTER\x10\x0c\x12\x0b\n\x07\x46LAG_EW\x10\r\x12\x0b\n\x07\x46LAG_DW\x10\x0e\x12\x0c\n\x08\x42\x43OUNTER\x10\x0f\x42&\n\x18\x65u.antidotedb.antidotepbB\nAntidotePB'
)

_CRDT_TYPE = _descriptor.EnumDescriptor(
  name='CRDT_type',
  full_name='CRDT_type',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='COUNTER', index=0, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ORSET', index=1, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LWWREG', index=2, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MVREG', index=3, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GMAP', index=4, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RWSET', index=5, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RRMAP', index=6, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FATCOUNTER', index=7, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FLAG_EW', index=8, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FLAG_DW', index=9, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BCOUNTER', index=10, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2800,
  serialized_end=2946,
)
_sym_db.RegisterEnumDescriptor(_CRDT_TYPE)

CRDT_type = enum_type_wrapper.EnumTypeWrapper(_CRDT_TYPE)
COUNTER = 3
ORSET = 4
LWWREG = 5
MVREG = 6
GMAP = 8
RWSET = 10
RRMAP = 11
FATCOUNTER = 12
FLAG_EW = 13
FLAG_DW = 14
BCOUNTER = 15


_APBSETUPDATE_SETOPTYPE = _descriptor.EnumDescriptor(
  name='SetOpType',
  full_name='ApbSetUpdate.SetOpType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ADD', index=0, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REMOVE', index=1, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=221,
  serialized_end=253,
)
_sym_db.RegisterEnumDescriptor(_APBSETUPDATE_SETOPTYPE)


_APBERRORRESP = _descriptor.Descriptor(
  name='ApbErrorResp',
  full_name='ApbErrorResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='errmsg', full_name='ApbErrorResp.errmsg', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errcode', full_name='ApbErrorResp.errcode', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18,
  serialized_end=65,
)


_APBCOUNTERUPDATE = _descriptor.Descriptor(
  name='ApbCounterUpdate',
  full_name='ApbCounterUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='inc', full_name='ApbCounterUpdate.inc', index=0,
      number=1, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=67,
  serialized_end=98,
)


_APBGETCOUNTERRESP = _descriptor.Descriptor(
  name='ApbGetCounterResp',
  full_name='ApbGetCounterResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ApbGetCounterResp.value', index=0,
      number=1, type=17, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=100,
  serialized_end=134,
)


_APBSETUPDATE = _descriptor.Descriptor(
  name='ApbSetUpdate',
  full_name='ApbSetUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='optype', full_name='ApbSetUpdate.optype', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adds', full_name='ApbSetUpdate.adds', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rems', full_name='ApbSetUpdate.rems', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _APBSETUPDATE_SETOPTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=136,
  serialized_end=253,
)


_APBGETSETRESP = _descriptor.Descriptor(
  name='ApbGetSetResp',
  full_name='ApbGetSetResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ApbGetSetResp.value', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=255,
  serialized_end=285,
)


_APBREGUPDATE = _descriptor.Descriptor(
  name='ApbRegUpdate',
  full_name='ApbRegUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ApbRegUpdate.value', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=287,
  serialized_end=316,
)


_APBGETREGRESP = _descriptor.Descriptor(
  name='ApbGetRegResp',
  full_name='ApbGetRegResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ApbGetRegResp.value', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=318,
  serialized_end=348,
)


_APBGETMVREGRESP = _descriptor.Descriptor(
  name='ApbGetMVRegResp',
  full_name='ApbGetMVRegResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='ApbGetMVRegResp.values', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=350,
  serialized_end=383,
)


_APBMAPKEY = _descriptor.Descriptor(
  name='ApbMapKey',
  full_name='ApbMapKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ApbMapKey.key', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='ApbMapKey.type', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=385,
  serialized_end=435,
)


_APBMAPUPDATE = _descriptor.Descriptor(
  name='ApbMapUpdate',
  full_name='ApbMapUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='updates', full_name='ApbMapUpdate.updates', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='removedKeys', full_name='ApbMapUpdate.removedKeys', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=437,
  serialized_end=522,
)


_APBMAPNESTEDUPDATE = _descriptor.Descriptor(
  name='ApbMapNestedUpdate',
  full_name='ApbMapNestedUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ApbMapNestedUpdate.key', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='update', full_name='ApbMapNestedUpdate.update', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=524,
  serialized_end=606,
)


_APBGETMAPRESP = _descriptor.Descriptor(
  name='ApbGetMapResp',
  full_name='ApbGetMapResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='entries', full_name='ApbGetMapResp.entries', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=608,
  serialized_end=654,
)


_APBMAPENTRY = _descriptor.Descriptor(
  name='ApbMapEntry',
  full_name='ApbMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ApbMapEntry.key', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ApbMapEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=656,
  serialized_end=729,
)


_APBFLAGUPDATE = _descriptor.Descriptor(
  name='ApbFlagUpdate',
  full_name='ApbFlagUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ApbFlagUpdate.value', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=731,
  serialized_end=761,
)


_APBGETFLAGRESP = _descriptor.Descriptor(
  name='ApbGetFlagResp',
  full_name='ApbGetFlagResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ApbGetFlagResp.value', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=763,
  serialized_end=794,
)


_APBCRDTRESET = _descriptor.Descriptor(
  name='ApbCrdtReset',
  full_name='ApbCrdtReset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=796,
  serialized_end=810,
)


_APBOPERATIONRESP = _descriptor.Descriptor(
  name='ApbOperationResp',
  full_name='ApbOperationResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ApbOperationResp.success', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorcode', full_name='ApbOperationResp.errorcode', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=812,
  serialized_end=866,
)


_APBTXNPROPERTIES = _descriptor.Descriptor(
  name='ApbTxnProperties',
  full_name='ApbTxnProperties',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='read_write', full_name='ApbTxnProperties.read_write', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='red_blue', full_name='ApbTxnProperties.red_blue', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shared_locks', full_name='ApbTxnProperties.shared_locks', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exclusive_locks', full_name='ApbTxnProperties.exclusive_locks', index=3,
      number=4, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=868,
  serialized_end=971,
)


_APBBOUNDOBJECT = _descriptor.Descriptor(
  name='ApbBoundObject',
  full_name='ApbBoundObject',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ApbBoundObject.key', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='ApbBoundObject.type', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bucket', full_name='ApbBoundObject.bucket', index=2,
      number=3, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=973,
  serialized_end=1044,
)


_APBREADOBJECTS = _descriptor.Descriptor(
  name='ApbReadObjects',
  full_name='ApbReadObjects',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='boundobjects', full_name='ApbReadObjects.boundobjects', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transaction_descriptor', full_name='ApbReadObjects.transaction_descriptor', index=1,
      number=2, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1046,
  serialized_end=1133,
)


_APBUPDATEOP = _descriptor.Descriptor(
  name='ApbUpdateOp',
  full_name='ApbUpdateOp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='boundobject', full_name='ApbUpdateOp.boundobject', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operation', full_name='ApbUpdateOp.operation', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1135,
  serialized_end=1226,
)


_APBUPDATEOPERATION = _descriptor.Descriptor(
  name='ApbUpdateOperation',
  full_name='ApbUpdateOperation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='counterop', full_name='ApbUpdateOperation.counterop', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='setop', full_name='ApbUpdateOperation.setop', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='regop', full_name='ApbUpdateOperation.regop', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mapop', full_name='ApbUpdateOperation.mapop', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resetop', full_name='ApbUpdateOperation.resetop', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='flagop', full_name='ApbUpdateOperation.flagop', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1229,
  serialized_end=1441,
)


_APBUPDATEOBJECTS = _descriptor.Descriptor(
  name='ApbUpdateObjects',
  full_name='ApbUpdateObjects',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='updates', full_name='ApbUpdateObjects.updates', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transaction_descriptor', full_name='ApbUpdateObjects.transaction_descriptor', index=1,
      number=2, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1443,
  serialized_end=1524,
)


_APBSTARTTRANSACTION = _descriptor.Descriptor(
  name='ApbStartTransaction',
  full_name='ApbStartTransaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='ApbStartTransaction.timestamp', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='ApbStartTransaction.properties', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1526,
  serialized_end=1605,
)


_APBABORTTRANSACTION = _descriptor.Descriptor(
  name='ApbAbortTransaction',
  full_name='ApbAbortTransaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transaction_descriptor', full_name='ApbAbortTransaction.transaction_descriptor', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1607,
  serialized_end=1660,
)


_APBCOMMITTRANSACTION = _descriptor.Descriptor(
  name='ApbCommitTransaction',
  full_name='ApbCommitTransaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transaction_descriptor', full_name='ApbCommitTransaction.transaction_descriptor', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1662,
  serialized_end=1716,
)


_APBSTATICUPDATEOBJECTS = _descriptor.Descriptor(
  name='ApbStaticUpdateObjects',
  full_name='ApbStaticUpdateObjects',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transaction', full_name='ApbStaticUpdateObjects.transaction', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='updates', full_name='ApbStaticUpdateObjects.updates', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1718,
  serialized_end=1816,
)


_APBSTATICREADOBJECTS = _descriptor.Descriptor(
  name='ApbStaticReadObjects',
  full_name='ApbStaticReadObjects',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transaction', full_name='ApbStaticReadObjects.transaction', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='objects', full_name='ApbStaticReadObjects.objects', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1818,
  serialized_end=1917,
)


_APBSTARTTRANSACTIONRESP = _descriptor.Descriptor(
  name='ApbStartTransactionResp',
  full_name='ApbStartTransactionResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ApbStartTransactionResp.success', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transaction_descriptor', full_name='ApbStartTransactionResp.transaction_descriptor', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorcode', full_name='ApbStartTransactionResp.errorcode', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1919,
  serialized_end=2012,
)


_APBREADOBJECTRESP = _descriptor.Descriptor(
  name='ApbReadObjectResp',
  full_name='ApbReadObjectResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='counter', full_name='ApbReadObjectResp.counter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='set', full_name='ApbReadObjectResp.set', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reg', full_name='ApbReadObjectResp.reg', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mvreg', full_name='ApbReadObjectResp.mvreg', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='map', full_name='ApbReadObjectResp.map', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='flag', full_name='ApbReadObjectResp.flag', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2015,
  serialized_end=2222,
)


_APBREADOBJECTSRESP = _descriptor.Descriptor(
  name='ApbReadObjectsResp',
  full_name='ApbReadObjectsResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ApbReadObjectsResp.success', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='objects', full_name='ApbReadObjectsResp.objects', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorcode', full_name='ApbReadObjectsResp.errorcode', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2224,
  serialized_end=2317,
)


_APBCOMMITRESP = _descriptor.Descriptor(
  name='ApbCommitResp',
  full_name='ApbCommitResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ApbCommitResp.success', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='commit_time', full_name='ApbCommitResp.commit_time', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorcode', full_name='ApbCommitResp.errorcode', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2319,
  serialized_end=2391,
)


_APBSTATICREADOBJECTSRESP = _descriptor.Descriptor(
  name='ApbStaticReadObjectsResp',
  full_name='ApbStaticReadObjectsResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='objects', full_name='ApbStaticReadObjectsResp.objects', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='committime', full_name='ApbStaticReadObjectsResp.committime', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2393,
  serialized_end=2493,
)


_APBCREATEDC = _descriptor.Descriptor(
  name='ApbCreateDC',
  full_name='ApbCreateDC',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='nodes', full_name='ApbCreateDC.nodes', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2495,
  serialized_end=2523,
)


_APBCREATEDCRESP = _descriptor.Descriptor(
  name='ApbCreateDCResp',
  full_name='ApbCreateDCResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ApbCreateDCResp.success', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorcode', full_name='ApbCreateDCResp.errorcode', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2525,
  serialized_end=2578,
)


_APBGETCONNECTIONDESCRIPTOR = _descriptor.Descriptor(
  name='ApbGetConnectionDescriptor',
  full_name='ApbGetConnectionDescriptor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2580,
  serialized_end=2608,
)


_APBGETCONNECTIONDESCRIPTORRESP = _descriptor.Descriptor(
  name='ApbGetConnectionDescriptorResp',
  full_name='ApbGetConnectionDescriptorResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ApbGetConnectionDescriptorResp.success', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='descriptor', full_name='ApbGetConnectionDescriptorResp.descriptor', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorcode', full_name='ApbGetConnectionDescriptorResp.errorcode', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2610,
  serialized_end=2698,
)


_APBCONNECTTODCS = _descriptor.Descriptor(
  name='ApbConnectToDCs',
  full_name='ApbConnectToDCs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='descriptors', full_name='ApbConnectToDCs.descriptors', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2700,
  serialized_end=2738,
)


_APBCONNECTTODCSRESP = _descriptor.Descriptor(
  name='ApbConnectToDCsResp',
  full_name='ApbConnectToDCsResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ApbConnectToDCsResp.success', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorcode', full_name='ApbConnectToDCsResp.errorcode', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2740,
  serialized_end=2797,
)

_APBSETUPDATE.fields_by_name['optype'].enum_type = _APBSETUPDATE_SETOPTYPE
_APBSETUPDATE_SETOPTYPE.containing_type = _APBSETUPDATE
_APBMAPKEY.fields_by_name['type'].enum_type = _CRDT_TYPE
_APBMAPUPDATE.fields_by_name['updates'].message_type = _APBMAPNESTEDUPDATE
_APBMAPUPDATE.fields_by_name['removedKeys'].message_type = _APBMAPKEY
_APBMAPNESTEDUPDATE.fields_by_name['key'].message_type = _APBMAPKEY
_APBMAPNESTEDUPDATE.fields_by_name['update'].message_type = _APBUPDATEOPERATION
_APBGETMAPRESP.fields_by_name['entries'].message_type = _APBMAPENTRY
_APBMAPENTRY.fields_by_name['key'].message_type = _APBMAPKEY
_APBMAPENTRY.fields_by_name['value'].message_type = _APBREADOBJECTRESP
_APBBOUNDOBJECT.fields_by_name['type'].enum_type = _CRDT_TYPE
_APBREADOBJECTS.fields_by_name['boundobjects'].message_type = _APBBOUNDOBJECT
_APBUPDATEOP.fields_by_name['boundobject'].message_type = _APBBOUNDOBJECT
_APBUPDATEOP.fields_by_name['operation'].message_type = _APBUPDATEOPERATION
_APBUPDATEOPERATION.fields_by_name['counterop'].message_type = _APBCOUNTERUPDATE
_APBUPDATEOPERATION.fields_by_name['setop'].message_type = _APBSETUPDATE
_APBUPDATEOPERATION.fields_by_name['regop'].message_type = _APBREGUPDATE
_APBUPDATEOPERATION.fields_by_name['mapop'].message_type = _APBMAPUPDATE
_APBUPDATEOPERATION.fields_by_name['resetop'].message_type = _APBCRDTRESET
_APBUPDATEOPERATION.fields_by_name['flagop'].message_type = _APBFLAGUPDATE
_APBUPDATEOBJECTS.fields_by_name['updates'].message_type = _APBUPDATEOP
_APBSTARTTRANSACTION.fields_by_name['properties'].message_type = _APBTXNPROPERTIES
_APBSTATICUPDATEOBJECTS.fields_by_name['transaction'].message_type = _APBSTARTTRANSACTION
_APBSTATICUPDATEOBJECTS.fields_by_name['updates'].message_type = _APBUPDATEOP
_APBSTATICREADOBJECTS.fields_by_name['transaction'].message_type = _APBSTARTTRANSACTION
_APBSTATICREADOBJECTS.fields_by_name['objects'].message_type = _APBBOUNDOBJECT
_APBREADOBJECTRESP.fields_by_name['counter'].message_type = _APBGETCOUNTERRESP
_APBREADOBJECTRESP.fields_by_name['set'].message_type = _APBGETSETRESP
_APBREADOBJECTRESP.fields_by_name['reg'].message_type = _APBGETREGRESP
_APBREADOBJECTRESP.fields_by_name['mvreg'].message_type = _APBGETMVREGRESP
_APBREADOBJECTRESP.fields_by_name['map'].message_type = _APBGETMAPRESP
_APBREADOBJECTRESP.fields_by_name['flag'].message_type = _APBGETFLAGRESP
_APBREADOBJECTSRESP.fields_by_name['objects'].message_type = _APBREADOBJECTRESP
_APBSTATICREADOBJECTSRESP.fields_by_name['objects'].message_type = _APBREADOBJECTSRESP
_APBSTATICREADOBJECTSRESP.fields_by_name['committime'].message_type = _APBCOMMITRESP
DESCRIPTOR.message_types_by_name['ApbErrorResp'] = _APBERRORRESP
DESCRIPTOR.message_types_by_name['ApbCounterUpdate'] = _APBCOUNTERUPDATE
DESCRIPTOR.message_types_by_name['ApbGetCounterResp'] = _APBGETCOUNTERRESP
DESCRIPTOR.message_types_by_name['ApbSetUpdate'] = _APBSETUPDATE
DESCRIPTOR.message_types_by_name['ApbGetSetResp'] = _APBGETSETRESP
DESCRIPTOR.message_types_by_name['ApbRegUpdate'] = _APBREGUPDATE
DESCRIPTOR.message_types_by_name['ApbGetRegResp'] = _APBGETREGRESP
DESCRIPTOR.message_types_by_name['ApbGetMVRegResp'] = _APBGETMVREGRESP
DESCRIPTOR.message_types_by_name['ApbMapKey'] = _APBMAPKEY
DESCRIPTOR.message_types_by_name['ApbMapUpdate'] = _APBMAPUPDATE
DESCRIPTOR.message_types_by_name['ApbMapNestedUpdate'] = _APBMAPNESTEDUPDATE
DESCRIPTOR.message_types_by_name['ApbGetMapResp'] = _APBGETMAPRESP
DESCRIPTOR.message_types_by_name['ApbMapEntry'] = _APBMAPENTRY
DESCRIPTOR.message_types_by_name['ApbFlagUpdate'] = _APBFLAGUPDATE
DESCRIPTOR.message_types_by_name['ApbGetFlagResp'] = _APBGETFLAGRESP
DESCRIPTOR.message_types_by_name['ApbCrdtReset'] = _APBCRDTRESET
DESCRIPTOR.message_types_by_name['ApbOperationResp'] = _APBOPERATIONRESP
DESCRIPTOR.message_types_by_name['ApbTxnProperties'] = _APBTXNPROPERTIES
DESCRIPTOR.message_types_by_name['ApbBoundObject'] = _APBBOUNDOBJECT
DESCRIPTOR.message_types_by_name['ApbReadObjects'] = _APBREADOBJECTS
DESCRIPTOR.message_types_by_name['ApbUpdateOp'] = _APBUPDATEOP
DESCRIPTOR.message_types_by_name['ApbUpdateOperation'] = _APBUPDATEOPERATION
DESCRIPTOR.message_types_by_name['ApbUpdateObjects'] = _APBUPDATEOBJECTS
DESCRIPTOR.message_types_by_name['ApbStartTransaction'] = _APBSTARTTRANSACTION
DESCRIPTOR.message_types_by_name['ApbAbortTransaction'] = _APBABORTTRANSACTION
DESCRIPTOR.message_types_by_name['ApbCommitTransaction'] = _APBCOMMITTRANSACTION
DESCRIPTOR.message_types_by_name['ApbStaticUpdateObjects'] = _APBSTATICUPDATEOBJECTS
DESCRIPTOR.message_types_by_name['ApbStaticReadObjects'] = _APBSTATICREADOBJECTS
DESCRIPTOR.message_types_by_name['ApbStartTransactionResp'] = _APBSTARTTRANSACTIONRESP
DESCRIPTOR.message_types_by_name['ApbReadObjectResp'] = _APBREADOBJECTRESP
DESCRIPTOR.message_types_by_name['ApbReadObjectsResp'] = _APBREADOBJECTSRESP
DESCRIPTOR.message_types_by_name['ApbCommitResp'] = _APBCOMMITRESP
DESCRIPTOR.message_types_by_name['ApbStaticReadObjectsResp'] = _APBSTATICREADOBJECTSRESP
DESCRIPTOR.message_types_by_name['ApbCreateDC'] = _APBCREATEDC
DESCRIPTOR.message_types_by_name['ApbCreateDCResp'] = _APBCREATEDCRESP
DESCRIPTOR.message_types_by_name['ApbGetConnectionDescriptor'] = _APBGETCONNECTIONDESCRIPTOR
DESCRIPTOR.message_types_by_name['ApbGetConnectionDescriptorResp'] = _APBGETCONNECTIONDESCRIPTORRESP
DESCRIPTOR.message_types_by_name['ApbConnectToDCs'] = _APBCONNECTTODCS
DESCRIPTOR.message_types_by_name['ApbConnectToDCsResp'] = _APBCONNECTTODCSRESP
DESCRIPTOR.enum_types_by_name['CRDT_type'] = _CRDT_TYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ApbErrorResp = _reflection.GeneratedProtocolMessageType('ApbErrorResp', (_message.Message,), {
  'DESCRIPTOR' : _APBERRORRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbErrorResp)
  })
_sym_db.RegisterMessage(ApbErrorResp)

ApbCounterUpdate = _reflection.GeneratedProtocolMessageType('ApbCounterUpdate', (_message.Message,), {
  'DESCRIPTOR' : _APBCOUNTERUPDATE,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbCounterUpdate)
  })
_sym_db.RegisterMessage(ApbCounterUpdate)

ApbGetCounterResp = _reflection.GeneratedProtocolMessageType('ApbGetCounterResp', (_message.Message,), {
  'DESCRIPTOR' : _APBGETCOUNTERRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbGetCounterResp)
  })
_sym_db.RegisterMessage(ApbGetCounterResp)

ApbSetUpdate = _reflection.GeneratedProtocolMessageType('ApbSetUpdate', (_message.Message,), {
  'DESCRIPTOR' : _APBSETUPDATE,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbSetUpdate)
  })
_sym_db.RegisterMessage(ApbSetUpdate)

ApbGetSetResp = _reflection.GeneratedProtocolMessageType('ApbGetSetResp', (_message.Message,), {
  'DESCRIPTOR' : _APBGETSETRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbGetSetResp)
  })
_sym_db.RegisterMessage(ApbGetSetResp)

ApbRegUpdate = _reflection.GeneratedProtocolMessageType('ApbRegUpdate', (_message.Message,), {
  'DESCRIPTOR' : _APBREGUPDATE,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbRegUpdate)
  })
_sym_db.RegisterMessage(ApbRegUpdate)

ApbGetRegResp = _reflection.GeneratedProtocolMessageType('ApbGetRegResp', (_message.Message,), {
  'DESCRIPTOR' : _APBGETREGRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbGetRegResp)
  })
_sym_db.RegisterMessage(ApbGetRegResp)

ApbGetMVRegResp = _reflection.GeneratedProtocolMessageType('ApbGetMVRegResp', (_message.Message,), {
  'DESCRIPTOR' : _APBGETMVREGRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbGetMVRegResp)
  })
_sym_db.RegisterMessage(ApbGetMVRegResp)

ApbMapKey = _reflection.GeneratedProtocolMessageType('ApbMapKey', (_message.Message,), {
  'DESCRIPTOR' : _APBMAPKEY,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbMapKey)
  })
_sym_db.RegisterMessage(ApbMapKey)

ApbMapUpdate = _reflection.GeneratedProtocolMessageType('ApbMapUpdate', (_message.Message,), {
  'DESCRIPTOR' : _APBMAPUPDATE,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbMapUpdate)
  })
_sym_db.RegisterMessage(ApbMapUpdate)

ApbMapNestedUpdate = _reflection.GeneratedProtocolMessageType('ApbMapNestedUpdate', (_message.Message,), {
  'DESCRIPTOR' : _APBMAPNESTEDUPDATE,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbMapNestedUpdate)
  })
_sym_db.RegisterMessage(ApbMapNestedUpdate)

ApbGetMapResp = _reflection.GeneratedProtocolMessageType('ApbGetMapResp', (_message.Message,), {
  'DESCRIPTOR' : _APBGETMAPRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbGetMapResp)
  })
_sym_db.RegisterMessage(ApbGetMapResp)

ApbMapEntry = _reflection.GeneratedProtocolMessageType('ApbMapEntry', (_message.Message,), {
  'DESCRIPTOR' : _APBMAPENTRY,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbMapEntry)
  })
_sym_db.RegisterMessage(ApbMapEntry)

ApbFlagUpdate = _reflection.GeneratedProtocolMessageType('ApbFlagUpdate', (_message.Message,), {
  'DESCRIPTOR' : _APBFLAGUPDATE,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbFlagUpdate)
  })
_sym_db.RegisterMessage(ApbFlagUpdate)

ApbGetFlagResp = _reflection.GeneratedProtocolMessageType('ApbGetFlagResp', (_message.Message,), {
  'DESCRIPTOR' : _APBGETFLAGRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbGetFlagResp)
  })
_sym_db.RegisterMessage(ApbGetFlagResp)

ApbCrdtReset = _reflection.GeneratedProtocolMessageType('ApbCrdtReset', (_message.Message,), {
  'DESCRIPTOR' : _APBCRDTRESET,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbCrdtReset)
  })
_sym_db.RegisterMessage(ApbCrdtReset)

ApbOperationResp = _reflection.GeneratedProtocolMessageType('ApbOperationResp', (_message.Message,), {
  'DESCRIPTOR' : _APBOPERATIONRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbOperationResp)
  })
_sym_db.RegisterMessage(ApbOperationResp)

ApbTxnProperties = _reflection.GeneratedProtocolMessageType('ApbTxnProperties', (_message.Message,), {
  'DESCRIPTOR' : _APBTXNPROPERTIES,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbTxnProperties)
  })
_sym_db.RegisterMessage(ApbTxnProperties)

ApbBoundObject = _reflection.GeneratedProtocolMessageType('ApbBoundObject', (_message.Message,), {
  'DESCRIPTOR' : _APBBOUNDOBJECT,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbBoundObject)
  })
_sym_db.RegisterMessage(ApbBoundObject)

ApbReadObjects = _reflection.GeneratedProtocolMessageType('ApbReadObjects', (_message.Message,), {
  'DESCRIPTOR' : _APBREADOBJECTS,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbReadObjects)
  })
_sym_db.RegisterMessage(ApbReadObjects)

ApbUpdateOp = _reflection.GeneratedProtocolMessageType('ApbUpdateOp', (_message.Message,), {
  'DESCRIPTOR' : _APBUPDATEOP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbUpdateOp)
  })
_sym_db.RegisterMessage(ApbUpdateOp)

ApbUpdateOperation = _reflection.GeneratedProtocolMessageType('ApbUpdateOperation', (_message.Message,), {
  'DESCRIPTOR' : _APBUPDATEOPERATION,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbUpdateOperation)
  })
_sym_db.RegisterMessage(ApbUpdateOperation)

ApbUpdateObjects = _reflection.GeneratedProtocolMessageType('ApbUpdateObjects', (_message.Message,), {
  'DESCRIPTOR' : _APBUPDATEOBJECTS,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbUpdateObjects)
  })
_sym_db.RegisterMessage(ApbUpdateObjects)

ApbStartTransaction = _reflection.GeneratedProtocolMessageType('ApbStartTransaction', (_message.Message,), {
  'DESCRIPTOR' : _APBSTARTTRANSACTION,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbStartTransaction)
  })
_sym_db.RegisterMessage(ApbStartTransaction)

ApbAbortTransaction = _reflection.GeneratedProtocolMessageType('ApbAbortTransaction', (_message.Message,), {
  'DESCRIPTOR' : _APBABORTTRANSACTION,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbAbortTransaction)
  })
_sym_db.RegisterMessage(ApbAbortTransaction)

ApbCommitTransaction = _reflection.GeneratedProtocolMessageType('ApbCommitTransaction', (_message.Message,), {
  'DESCRIPTOR' : _APBCOMMITTRANSACTION,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbCommitTransaction)
  })
_sym_db.RegisterMessage(ApbCommitTransaction)

ApbStaticUpdateObjects = _reflection.GeneratedProtocolMessageType('ApbStaticUpdateObjects', (_message.Message,), {
  'DESCRIPTOR' : _APBSTATICUPDATEOBJECTS,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbStaticUpdateObjects)
  })
_sym_db.RegisterMessage(ApbStaticUpdateObjects)

ApbStaticReadObjects = _reflection.GeneratedProtocolMessageType('ApbStaticReadObjects', (_message.Message,), {
  'DESCRIPTOR' : _APBSTATICREADOBJECTS,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbStaticReadObjects)
  })
_sym_db.RegisterMessage(ApbStaticReadObjects)

ApbStartTransactionResp = _reflection.GeneratedProtocolMessageType('ApbStartTransactionResp', (_message.Message,), {
  'DESCRIPTOR' : _APBSTARTTRANSACTIONRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbStartTransactionResp)
  })
_sym_db.RegisterMessage(ApbStartTransactionResp)

ApbReadObjectResp = _reflection.GeneratedProtocolMessageType('ApbReadObjectResp', (_message.Message,), {
  'DESCRIPTOR' : _APBREADOBJECTRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbReadObjectResp)
  })
_sym_db.RegisterMessage(ApbReadObjectResp)

ApbReadObjectsResp = _reflection.GeneratedProtocolMessageType('ApbReadObjectsResp', (_message.Message,), {
  'DESCRIPTOR' : _APBREADOBJECTSRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbReadObjectsResp)
  })
_sym_db.RegisterMessage(ApbReadObjectsResp)

ApbCommitResp = _reflection.GeneratedProtocolMessageType('ApbCommitResp', (_message.Message,), {
  'DESCRIPTOR' : _APBCOMMITRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbCommitResp)
  })
_sym_db.RegisterMessage(ApbCommitResp)

ApbStaticReadObjectsResp = _reflection.GeneratedProtocolMessageType('ApbStaticReadObjectsResp', (_message.Message,), {
  'DESCRIPTOR' : _APBSTATICREADOBJECTSRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbStaticReadObjectsResp)
  })
_sym_db.RegisterMessage(ApbStaticReadObjectsResp)

ApbCreateDC = _reflection.GeneratedProtocolMessageType('ApbCreateDC', (_message.Message,), {
  'DESCRIPTOR' : _APBCREATEDC,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbCreateDC)
  })
_sym_db.RegisterMessage(ApbCreateDC)

ApbCreateDCResp = _reflection.GeneratedProtocolMessageType('ApbCreateDCResp', (_message.Message,), {
  'DESCRIPTOR' : _APBCREATEDCRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbCreateDCResp)
  })
_sym_db.RegisterMessage(ApbCreateDCResp)

ApbGetConnectionDescriptor = _reflection.GeneratedProtocolMessageType('ApbGetConnectionDescriptor', (_message.Message,), {
  'DESCRIPTOR' : _APBGETCONNECTIONDESCRIPTOR,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbGetConnectionDescriptor)
  })
_sym_db.RegisterMessage(ApbGetConnectionDescriptor)

ApbGetConnectionDescriptorResp = _reflection.GeneratedProtocolMessageType('ApbGetConnectionDescriptorResp', (_message.Message,), {
  'DESCRIPTOR' : _APBGETCONNECTIONDESCRIPTORRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbGetConnectionDescriptorResp)
  })
_sym_db.RegisterMessage(ApbGetConnectionDescriptorResp)

ApbConnectToDCs = _reflection.GeneratedProtocolMessageType('ApbConnectToDCs', (_message.Message,), {
  'DESCRIPTOR' : _APBCONNECTTODCS,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbConnectToDCs)
  })
_sym_db.RegisterMessage(ApbConnectToDCs)

ApbConnectToDCsResp = _reflection.GeneratedProtocolMessageType('ApbConnectToDCsResp', (_message.Message,), {
  'DESCRIPTOR' : _APBCONNECTTODCSRESP,
  '__module__' : 'antidote_pb2'
  # @@protoc_insertion_point(class_scope:ApbConnectToDCsResp)
  })
_sym_db.RegisterMessage(ApbConnectToDCsResp)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jan 25 21:36:45 2019

@author: nmp
"""

import setuptools

with open("README.MD", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="antidotedb",
    version="0.1.3",
    author="Nuno Preguica",
    author_email="nuno.preguica@fct.unl.pt",
    description="AntidoteDB Python clients",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AntidoteDB/antidote-python-client/tree/master/src/main/python/antidotedb",
    packages=['antidotedb','antidotedb.proto'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
          'protobuf',
      ],
)
